
import java.time.Month;

import java.time.format.DateTimeFormatter;
import java.time.LocalDate;
import java.util.Locale;

import java.util.HashMap;
import java.util.Map;

import java.util.List;


public class Article {
	
	private String title;
	private String url;
	private List<String> author;
	private String section;
	private String description;
	private String text;
	
    /**
     * Most of articles from 'www.theguardian.com' will have the publication date that looks like this:
     *    
     *    2021-04-28T10:05:43.000Z
     * 	
     * However, it can happen that: (a) the publication date is missing or (b) it is not in this format.
     * The 'publicationDate' attribute is set to the default date '01.01.1900' as this is necessary to
     * avoid the GuardianSpider being trapped in an infinite loop.
     */
	DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'", Locale.ENGLISH);
	private LocalDate publicationDate = LocalDate.parse("1900-01-01T09:00:00.000Z", formatter);
	
	
	public Article() {
		
	}
	
	
	public void setTitle(String title) {
		
		this.title = title;
		
	}
	
	public void setText(String text) {
		
		this.text = text;
		
	}
	
	public void setAuthor(List<String> author) {
		
		this.author = author;
		
	}

	public List<String> getAuthor() {
	
		return author;
	
	}
	
	public void setSection(String section) {
		
		this.section = section;
		
		}
	
	public void setDescription(String description) {
		
		this.description = description;
		
		}
	
	public String getSection() {
		return section;
		
	}
	
	public String getText() {
		return text;
		
	}
	
	public String getDescription() {
		return description;
		
	}
	
	public void setUrl(String url) {
		
		this.url = url;
	}
	
	public String getUrl() {
		return url;
		
	}
	
	public LocalDate getPublicationDate() {
		
		return publicationDate;
	}
	
    /**
     * Set the publication date of the article. 
     * It falls back to the default value if:
     *  (a) the publication date is missing or 
     *  (b) it's in a different format than the one expected
     *
     * @param date
     */
	public void setPublicationDate(String date) {
		 
		 
		 if (date != null) {
			 
		 try {
			 LocalDate parsedDate = LocalDate.parse(date, formatter);
			 
			 int year = parsedDate.getYear();
			 Month month = parsedDate.getMonth();
			 int day = parsedDate.getDayOfMonth();
			 
			 this.publicationDate = this.publicationDate.withYear(year).withMonth(month.getValue()).withDayOfMonth(day);
			 
		 }
		 
		 catch (Exception e) {
			System.out.println("Failed parsing date: " + date);
		}
		 
		 }
	 }
	
    /**
     * Transform object into a 'HashMap'.
     * Optional.
     * Used only to avoid Gson writing into the JSON file the specs of the publication date format.
     * @param articleMap
     */
	public Map<String,String> toMap() {
		
		HashMap<String,String> articleMap = new HashMap<>();
		
		articleMap.put("url", url);
		
		articleMap.put("title",title);
		
		articleMap.put("publicationDate", publicationDate.toString());
		
		articleMap.put("section", section);
		
		articleMap.put("author", String.join(" ", author));
		
		articleMap.put("description", description);
		
		articleMap.put("text", text);
		
		return articleMap;
		
	}
	}
